#ifndef TRIM_CLASS_H
#define TRIM_CLASS_H
#include <array>
#include <string>
#include <vector>
//入射原子类
class particle {
public:
    unsigned atomic_number;//原子序数
    double atomic_mass;//相对原子质量（AMU）
    double energy;//能量（keV）
    std::array<double, 3> direction;//速度方向矢量
    std::array<double, 3> displacement;//位移
    //默认构造函数
    //默认初始速度方向{0，0，1}，位移{0，0，0}
    inline particle(const unsigned Z=1, const double M=1.0078, const double E=0) :
        atomic_number(Z), atomic_mass(M), energy(E), direction({ 0,0,1 }), displacement({ 0,0,0 }) {}
    //拷贝构造函数
    //用一个已知的particle构造一个一模一样的新的particle
    inline particle(const particle& other)
        :atomic_number(other.atomic_number), atomic_mass(other.atomic_mass), energy(other.energy),
        direction(other.direction), displacement(other.displacement) {}
    //拷贝操作符
    inline particle& operator=(const particle& other) {
        atomic_number = other.atomic_number;
        atomic_mass = other.atomic_mass;
        energy = other.energy;
        direction = other.direction;
        displacement = other.displacement;
        return *this;
    }
};
//靶组分
struct target_component {
    unsigned atomic_number=14;//原子序数
    double atomic_mass=28.086;//相对原子质量（AMU）
    unsigned int stoichiometry=1;//原子比例
    double relative_amount=1;//原子数百分比
    double displacment_energy=15;//位移能
    double lattice_energy=2;//栅格能
    double surface_energy=4.7;//表面能
};
//靶层
class target_layer {
public:
    unsigned number_component;//组分数
    double thickness;//厚度（Å）
    double density;//密度（g/cm^3）
    double average_atomic_number;//平均原子序数
    double average_atmoic_mass;//平均相对原子质量
    double average_number_density;//平均原子数密度
    double compound_corr;//混合修正系数
    std::vector<target_component> components;//组分
    //默认构造函数
    inline target_layer(const unsigned N=1, const double T=10000, const double Rho=2.3009)
        :number_component(N), thickness(T), density(Rho),average_atomic_number(0),average_atmoic_mass(0),
        average_number_density(0), compound_corr(1),components(N) {}
    //拷贝构造函数
    //用一个已知的target_layer构造一个一模一样的新的target_layer
    inline target_layer(const target_layer& other)
        :number_component(other.number_component), thickness(other.thickness),density(other.density),
        average_atomic_number(other.average_atomic_number),average_atmoic_mass(other.average_atmoic_mass),
        average_number_density(other.average_number_density),compound_corr(other.compound_corr),components(other.components) {}
    //拷贝操作符
    inline target_layer& operator=(const target_layer& other) {
        number_component = other.number_component;
        thickness = other.thickness;
        density = other.density;
        average_atomic_number = other.average_atomic_number;
        average_atmoic_mass = other.average_atmoic_mass;
        average_number_density = other.average_number_density;
        compound_corr=other.compound_corr;
        components = other.components;
        return *this;
    }
};
//靶类
class target {
public:
    unsigned number_layer;//层数
    std::vector<target_layer> layers;//片层
    std::vector<double> layers_depth;//片层深度
    //默认构造函数
    inline target(const unsigned N=1) :number_layer(N), layers(N),layers_depth(N) {}
    //拷贝构造函数
    inline target(const target& other) :number_layer(other.number_layer), layers(other.layers),layers_depth(other.layers_depth) {}
    //拷贝操作符
    inline target& operator=(const target& other) {
        number_layer = other.number_layer;
        layers = other.layers;
        layers_depth = other.layers_depth;
        return *this;
    }
};

/*
元素周期表元素结构
*/
struct element_ptable {
    std::string_view name_abbrev;//名称缩写
    std::string_view name;//名称
    unsigned int atomic_number;//原子序数
    unsigned int most_isotopic_mass;//最大丰度原子质量
    double most_isotopic_weight;//最大丰度相对原子质量
    double nature_isotopic_mass;//自然丰度相对原子质量
    double density;//密度(g/cm^3)
    double num_density;//数密度(atoms/cm^3)
    double fermi_velocity;//费米速度(eV)
};

//模拟参数
struct simulate_parameter {
    unsigned number_simulation=10000;//模拟次数
    unsigned number_drawing=100;
    double final_energy=20;//最终停止时的能量
    double minimum_transfered_energy=5;//最小转移能量
    double simulate_lowest_energy=10;//模拟最低能量
    double simulate_highest_energy=10000;//模拟最高能量
    unsigned method_theta_calculation=0;
};

//模拟结果
struct simulate_result{
    unsigned int num_stopped=0;//停止粒子
    unsigned int num_transmitted=0;//透射粒子
    unsigned int num_backscatted=0;//背向散射粒子
    double projected_range=0;//投影射程
    double range=0;//射程
    std::vector<double> x_record;
    std::vector<double> y_record;
    std::vector<double> z_record;
};

#endif // !TRIM_CLASS_H
